package ema.tpjee.banking.dao;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;


import ema.tpjee.banking.model.Bank;
import ema.tpjee.banking.model.Customer;
import ema.tpjee.utils.Paging;
import ema.tpjee.utils.RandomString;


@Stateless
public class BankDaoImpl implements BankDao, Serializable {


	private static final long serialVersionUID = -5443611226894567253L;

	@PersistenceContext(unitName = "bankunit")
	private EntityManager entityManager = null;

	public BankDaoImpl() {
		super();
		System.out.println("creating BankDaoImpl");
	}

	public Bank find(Bank b) {
		return entityManager.find(Bank.class, b.getId());
	}

	public Bank add(Bank b) {
		if (find(b) == null) {
			entityManager.persist(b);
			return b;
		} else
			return null;
	}

	public Bank clone(Bank b) {
		Bank newb = new Bank(b.getName(), b.getAddress(), b.getZipCode(), b
				.getPhone());
		entityManager.persist(newb);
		return newb;
	}

	public Bank create() {
		Bank newb = new Bank("noname");
		entityManager.persist(newb);
		return newb;
	}

	private Random rand = new Random();

	public Bank createRandom() {
		Bank b = new Bank();
		b.setName("Banque " + RandomString.generateStringAlpha(10));
		b.setAddress(rand.nextInt(100) + " rue "
				+ RandomString.generateStringAlpha(10));
		b.setPhone(rand.nextInt(100) + "." + rand.nextInt(100) + "."
				+ rand.nextInt(100) + "." + rand.nextInt(100) + "."
				+ rand.nextInt(100));
		b.setZipCode(Integer.toString(30000 + rand.nextInt(1000)));
		entityManager.persist(b);
		return b;
	}

	public Bank delete(Bank b) {
		Bank pr = prior(b);
		if (pr == null)
			pr = next(b);
		//TODO vrifier si c'est ncessaire de supprimer les customers d'une banque
		/*
		Collection<Customer> bcusts= b.getCustomers(); //est-ce ncessaire ??
		for (Customer customer : bcusts) {
			entityManager.remove(customer);
		}	*/
		entityManager.remove(find(b.getId()));
		return pr;
	}

	public Bank find(int id) {
		return entityManager.find(Bank.class, id);
	}

	public Bank first() {
		List<Bank> bks = getList();
		return bks.get(0);
	}

	public Bank next(Bank b) {
		return Paging.next(getList(),b);
	}

	public Bank prior(Bank b) {
		return Paging.prior(getList(),b);
	}

	@SuppressWarnings("unchecked")
	public List<Bank> getList() {
		return entityManager.createNamedQuery("allbanks").getResultList();
	}

	public Bank last() {
		List<Bank> bks = getList();
		return bks.get(bks.size() - 1);
	}



	public Bank foobar(Bank b) {
		// TODO Auto-generated method stub
		return b;
	}

	public Bank merge(Bank b) {
		if (find(b) != null) {
			entityManager.merge(b);
			return b;
		} else
			return null;
	}


	
	public List<Bank> populate() {
		List<Bank> bqs = BankingPopulator.populate();
		for (Bank b: bqs){
			entityManager.persist(b);	
		}
		
		/*
		 * City c1 = new City(0, "Paris", "75000", "France"); addCity(c1); City
		 * c2 = new City(1, "Marseille", "13000", "France"); addCity(c2);
		 */
		/*
		Bank b = new Bank("Credit Arboricole", "3 rue des sapins", "75000",
				"123456");
		entityManager.persist(b);
		b = new Bank("Internet Bank", "10, avenue de la Californie", "13000",
				"7890123");
		entityManager.persist(b);
		b = new Bank("Socit Gniale", "1, parvis de la Dfonse", "92400",
				"45698741");
				*/

		return getList();
	}
}
