package test;

import java.util.Collection;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import ema.tpjee.banking.model.Bank;
import ema.tpjee.banking.model.Customer;

/**
 * Bean sans etat assurant le dialogue cote serveur avec les beans entites.
 * 
 * @author pfister
 */
@Stateless
public class TestFacade implements TestFacadeRemote {

	/**
	 * Entity manager utilise par ce bean.
	 */
	@PersistenceContext(unitName = "bankunit")
	private EntityManager entityManager = null;

	public Bank find(Bank b) {
		return entityManager.find(Bank.class, b.getId());
	}

	public void add(Bank b) {
		if (find(b) == null)
			entityManager.persist(b);
	}

	public void populate() {
		List<Bank> bqs = getBanks();
		if (bqs.size() > 2)
			return;
		/*
		 * City c1 = new City(0, "Paris", "75000", "France"); addCity(c1); City
		 * c2 = new City(1, "Marseille", "13000", "France"); addCity(c2);
		 */

		Bank b = new Bank("Credit Arboricole", "3 rue des sapins", "75000",
				"5646456");

		b.addCustomer(new Customer("Dupont", "Jean", "10, rue des lilas",
				"30000"));

		b.addCustomer(new Customer("Poulain", "Amlie", "20, rue des mimosas",
				"34000"));

		entityManager.persist(b);

		b = new Bank("Internet Bank", "10, avenue de la Californie", "13000",
				"5646456");
		b.addCustomer(new Customer("Durand", "Jacques",
				"10, rue des violettes", "32000"));

		b.addCustomer(new Customer("Harry", "Cover", "20, rue des salsifis",
				"31000"));

		entityManager.persist(b);

		b = new Bank("Socit Gniale", "1, parvis de la Dfonse", "92400",
				"5646456");

		// should be done by the server
		entityManager.joinTransaction();
	}

	/**
	 * @return la liste des Banks
	 */
	@SuppressWarnings("unchecked")
	public List<Bank> getBanks() {
		return entityManager.createNamedQuery("allbanks").getResultList();
	}

	/**
	 * @return la liste des Customers
	 */
	@SuppressWarnings("unchecked")
	public List<Customer> getCustomers() {
		return entityManager.createNamedQuery("allcustomers").getResultList();
	}

	public void delete(Bank b) {
		// TODO vrifier si c'est ncessaire de supprimer les customers d'une
		// banque

		b = entityManager.find(Bank.class, b.getId());
		/*
		 * Collection<Customer> bcusts = b.getCustomers(); //est-ce ncessaire ??
		 * for (Customer customer : bcusts) { entityManager.remove(customer); }
		 */
		entityManager.remove(b);
	}

	public void delete(Customer c) {
		c = entityManager.find(c.getClass(), c.getId());
		c.getBank().getCustomers().remove(c);
		entityManager.remove(c);
	}

}